<?php
/*
* Copyright (C) 1999-2000 EKILAT LLC (http://ekilat.com - the first reactor)
* 
* php(REACTOR) is open source; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License <http://www.opensource.org/licenses/gpl-license.html>
* for a complete version of the license.
*
**			./inc/users.inc.php
* this file contains all the functions for validating users, showing login
* status, etc.
* SPECIAL: THIRD PARTY WEBMAIL
* we have incorporated a facility for retreiving session information from third
* party webmail services. Basically,
* you can incorporate any third party webmail service which:
* - stores session information in cookies
* - is customizable enough for you to put an <img> tag (which points to your main
*   webserver) into their webmail interface.
* - running a webmail service is not a trivial operation.  people get _VERY_
*   upset when they cannot reach their mail.  that's why we leave this to the
*   third party.
* the webmail login here is defaulted to our service at http://webmail.ekilat.com 
* you'll have to make the necessary changes if you want to use it.
* BEWARE THE PROXY SERVER!
*
* to customize this to incorporate existing users databases, just update the
* respective functions.
*/

// tell everyone we're already loaded
define("REACTOR_INC_USERS", 1);

//include files that we'll need - check first
if(!defined("REACTOR_INC_GLOBAL")) { include("inc/global.inc.php"); }
if(!defined("REACTOR_INC_DB")) { include($pathtohomedir."/inc/db.inc.php"); }
if(!defined("REACTOR_INC_LANG")) { include($pathtohomedir."/inc/lang.inc.php"); }

//INCLUDE CONF
if(!defined("REACTOR_CONF_USERS")) { include($pathtohomedir."/conf/users.conf.php"); }


//specify the database that the users table is located in
######## v.1.0.0
function users_query($sql, $mode) {
	return db1_query($sql, $mode);
}

/*
* usernameformat() - checks to see if the username is a valid choice
* letters, numbers, ., _ (like an email address) 2 to 20 characters
*/
######## v.1.0.0
function usernameformat($username) {
	if(!ereg("^[[:alnum:]]{1}[[:alnum:]\._]{1,19}$", $username)) 
		return false;
	else return true;
}

/*
* passwordformat() - checks to see if the password is a valid choice
* letters or numbers, 4 to 20 characters.  
*/
######## v.1.0.0
function passwordformat($password) {
	if(!ereg("^[[:alnum:]]{4,20}$", $password)) 
		return false;
	else return true;
}

/*
* adduser() - adds a user into the system.  assumes that user submitted content
* has been tested for bad input, and verified for existence - we leave these
* functions to the forms because the forms have to react to them.  if the user
* can't be added, set the $errormessage variable and return false.
* uses "w" permissions
* NOTE about the encryption of passwords in the database.  OK, ENCODE and DECODE
* are NOT secure encryption.  They are only a hindrance.  Your database should
* already be secured with a username and password.  The reason why I use this
* is that I wanted a way to pull the passwords out of the database for a lost
* password function.
*/
######## v.1.0.0
function adduser(&$username, &$password, &$password2, &$email, &$profile) {
global $errormessage, $userstable, $userdb_private_key, $badwords, $userlang;
	
	$username = strtolower(trim($username));//always lowercase
	$profile = censor($profile, $badwords);
	if (!emailformat($email)) {//validate email
		$errormessage .= picklang($userlang["err_invalid_email"])."<br>\n";
		$email = "";
		return false;
	}
	else if($password != $password2) {//make sure passwords match
		$errormessage .= picklang($userlang["err_password_not_match"])."<br>\n";
		$password = "";
		$password2 = "";
		return false;
	}
	else if (!passwordformat($password)) {//validate password
		$errormessage .= picklang($userlang["err_password_bad_format"])."<br>\n";
		$password = "";
		$password2 = "";
		return false;
	}
	else if (!usernameformat($username)) {//validate username
		$errormessage .= picklang($userlang["err_username_bad_format"])."<br>\n";
		$username = "";
		return false;
	}
	//check to see if username is already taken
	$sql = "SELECT username FROM $userstable WHERE username='".$username."'";
	if ($request = users_query($sql,"w")) {
		$myuser = request_fetch_array($request);
		if($myuser["username"]) {
			$errormessage .= picklang($userlang["err_username_taken"])."<br>\n";
			$username = $username.date("Y");
			return false;
		}
	}
	//if we made it this far, all is well.  add the new user.
	$sql2 = "INSERT INTO $userstable (username, password, email, profile, posts, admin, status, modified, created) VALUES ('".$username."', ENCODE(\"$password\",\"".md5($userdb_private_key.$username)."\"),'".$email."', '".$profile."', 0,0,1,NULL,NULL)";
	if ($request2 = users_query($sql2,"w")) {
		return true;
	}
	else {
		$errormessage .= picklang($userlang["err_db_problem"])."<br>\n";
		return false;
	}
}

/*
* verifyuser() - this function validates the username and password against 
* the user's table.  if 
* the pair match, it returns the user's record (array form) from the user table.
* otherwise, sets $errormessage and returns false.
* uses "w" permissions because this function is usually coupled with updates
*/
######## v.1.0.0
function verifyuser(&$username, &$password, $bbs_login, $bbs_public_key, $returnto) {
global $reactorcore, $userstable, $errormessage, $userdb_private_key, $userlang;
$isloggedin = false;

	$username = strtolower(trim($username));//always lowercase
	if (bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//this means we're logged in
		//these don't do anything except cancel the error messages
		$username = $bbs_login;
		$password = $bbs_public_key;
		$isloggedin = true;
	}
	//make sure username and password are filled
	if($username == "") {
		$errormessage .= picklang($userlang["err_username_empty"])."<br>\n";
		return false;
	}
	else if($password == "") {
		$errormessage .= picklang($userlang["err_password_empty"])."<br>\n";
		return false;
	}
	//get their record from the database
	$sql = "SELECT *, DECODE(password,\"".md5($userdb_private_key.$username)."\") AS passwd FROM $userstable WHERE username='".$username."' AND status=1"; 
	if($request = users_query($sql,"w")) {
		$myuser = request_fetch_array($request);
		//if they are a valid user, return the array
		if ($isloggedin) return $myuser;
		//otherwise, see if the password validates
		else if ($password == $myuser["passwd"]) return $myuser;
		//doesn't validate? redirect to signup
		else {
			header("Location:".$reactorcore."/signup.php?badpass=1&returnto=".urlencode($returnto));
			exit;
		}
	}
	else $errormessage .= picklang($userlang["err_db_problem"])."<br>\n";
}

/*
* quickuserstatus() - this function sets the "status" of the user by looking
* at the admin and posts variables in the users table.  you can change the
* status descriptors.  if you want to change the levels, make the corresponding
* changes in setpostrank() and ispoweruser().
*/
######## v.1.0.0
function quickuserstatus($posts, $admin) {
global $errormessage;
	
	if ($admin == 2) $status = "administrator";		
	else if ($admin == 1) $status = "moderator";
	else if ($posts > 499) $status = "AI bot";
	else if ($posts > 74) $status = "respected member";
	else if ($posts > 19) $status = "member";
	else if ($posts > 2) $status = "candidate";
	else $status = "newbie";
	return $status;
}

/*
* setpostrank() - returns the initial rank of a users post based on previous
* posts and admin.  if you update quickuserstatus(), update this too.
*/
function setpostrank($useradmin, $userposts) {
	//determine rank from user status
	if (($userposts > 499) || ($useradmin == 2)) $rank = 3;
	else if (($userposts > 74) || ($useradmin == 1 )) $rank = 2;
	else if ($userposts > 19) $rank = 1;
	else $rank = 0;
	return $rank;
}

/*
* ispoweruser() - this function allows you to limit the advanced funtionality of
* your php(REACTOR); - for example, you may only want to allow proven users to
* post polls or request new forums.  (this will prevent abuse of the advanced
* functionality, while motivating new users to request new forums).
*/
function ispoweruser($useradmin, $userposts) {
global $errormessage, $userlang, $powerthreshold;
	if (!(($useradmin > 0) || ($userposts >= $powerthreshold))) {
		$errormessage .= picklang($userlang["not_power_user"])."<br>\n";
		return false;
	}
	else return true;
}

/*
* updateuserposts() - updates the posts column for a specific user.
*/
function updateuserposts($username, $add) {
global $userstable;
if ($add < 0) $sign = "";
else $sign = "+";
// echo $add;
		$sql = "UPDATE $userstable SET posts=posts".$sign.$add." WHERE username='".$username."'";
		if ($request = users_query($sql,"w")) {
			return true;
		}
		else return false;
}

/*
* userprofile() - show the information for this user
*/
######## v.1.0.0
function userprofile($username) {
global $reactorcore, $userlang, $userstable, $reactorcolor, $datestringshort, $PHP_SELF;

	//show basic information
	$sql = "SELECT email, profile, posts, admin, UNIX_TIMESTAMP(modified) AS last, UNIX_TIMESTAMP(created) AS first FROM $userstable WHERE username='".$username."' AND status=1";
	if ($request = users_query($sql,"r")) {
		if ($myuser = request_fetch_array($request)) {
			$outputhtml .= "<br><br>\n";
			$outputhtml .= mktable("90%", $reactorcolor[2], "6");
			$outputhtml .= "<tr>";
			$outputhtml .= "<td><h2>$username</h2><p>email: <a href=mailto:".$myuser["email"].">".$myuser["email"]."</a></p></td>";
			$outputhtml .= "<td><p><b>".quickuserstatus($myuser["posts"], $myuser["admin"]).": ".$myuser["posts"]." ".picklang($userlang["comments"])."</b></p></td>";
			$outputhtml .= "</tr>\n";
			$outputhtml .= "<tr>";
			$outputhtml .= "<td><p>".picklang($userlang["member_since"]).": ".date($datestringshort, $myuser["first"])."</p></td>";
			$outputhtml .= "<td><p>".picklang($userlang["last_post"]).": ".date($datestringshort, $myuser["last"])."</p></td>";
			$outputhtml .= "</tr>\n";
			$outputhtml .= "<tr>";
			$outputhtml .= "<td colspan=2><hr width=100% size=2 color=".$reactorcolor[0]."><p>".nl2br(makelinks($myuser["profile"]))."</p></td>";
			$outputhtml .= "</tr>\n";
			$outputhtml .= "<tr>";
			$outputhtml .= "<td colspan=2 align=right>[<a href=$reactorcore/editprofile.php?username=$username ".jpopclick("edit",400,434).">".picklang($userlang["change"])."</a>]</p></td>";
			$outputhtml .= "</tr>\n";
			$outputhtml .= "</table>\n";
			$outputhtml .= "<br>\n";
		}
	}
return $outputhtml;
}

/*
* bbs_login() just sets a cookie to let us know that the user is logged in
*/
######## v.1.0.0
function bbs_login($nick) {
global $bbs_private_key;
$path = "/";//available to all
$expires = "";//session cookie (in browser memory)
$domain = "";
$secure = 0;
$bbs_public_key = md5($nick.$bbs_private_key);

	setcookie("bbs_login", $nick, $expires, $path, $domain, $secure);
	setcookie("bbs_public_key", $bbs_public_key, $expires , $path, $domain, $secure);
	return $bbs_public_key;//maybe this will be useful.
}

/*
* bbs_login_confirm() checks the cookie against the private key to see if the 
* user is actually logged in
*/
######## v.1.0.0
function bbs_login_confirm($bbs_login, $bbs_public_key) {
global $bbs_private_key;
	if ($bbs_public_key == md5($bbs_login.$bbs_private_key))
		return $bbs_login;
	else return false;
}

/*
* bbs_login_box() for posting to the bbs - if the user is not logged in, it asks
* for a password. otherwise, it shows the logged in user's name with a logout
* link
*/
######## v.1.0.0
function bbs_login_box($bbs_login, $bbs_public_key) {
global $reactorcore, $username, $password, $userlang;

	if (bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//logged in
		?>
<tr><td align="right"><p><b>nick:</b></p></td><td width="80%"><b style="font-size:larger;"><?php echo $bbs_login;?></b> [<a href="javascript:opener.location='<?php echo $reactorcore; ?>/logout.php';window.close();"><?php echo picklang($userlang["logout"]); ?></a>]</td></tr>
		<?php	
	}
	else {
		//not logged in, prompt for nick and password
		?>
<tr><td align="right"><p><b>nick:</b></p></td><td width="80%"><input type="text" name="username" value="<?php echo $username;?>" size="12" maxlength="20"> <?php echo picklang($userlang["not_signed_up"]); ?> [<a href="<?php echo $reactorcore; ?>/signup.php" tabindex="100"><?php echo picklang($userlang["signup"]); ?></a>]</td></tr>
<tr><td align="right"><p><b>password:</b></p></td><td width="80%"><input type="password" name="password" value="<?php echo $password;?>" size="12" maxlength="20"> [<a href="<?php echo $reactorcore; ?>/forgot.php" tabindex="200"><?php echo picklang($userlang["forgot_password"]); ?></a>]</td></tr>
		<?php
	}
}

/*
* login_status() - this just provides a logout link if the user is logged in.
* USE THIS IF YOU ARE NOT USING A THIRD PARTY WEBMAIL SERVICE.
*/
######## v.1.0.0
function login_status($bbs_login, $bbs_public_key) {
global $reactorcore, $userlang;
	if (bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//reactor user is logged in
		?>
[<a href="<?php echo $reactorcore; ?>/logout.php"><?php echo picklang($userlang["logout"]); ?></a>]		
		<?php
	}
	else {
		?>
[not logged in]
		<?php
	}
}

/*
* email_login_confirm() - just checks to see if the cookie from the third party
* service is set - and has not expired.
*/
######## v.1.0.0
function email_login_confirm($email_login, $email_login_expires) {
	if((!isset($email_login)) || ($email_login_expires < time()))
		return false;
	else return true;
}

/*
* top_login_box() -if noone is logged in, displays login form for webmail.
* designed to be used with top.tpl.php.  THIS FUNCTION REQUIRES CUSTOMIZATION
* BEFORE YOU CAN USE IT. YOU HAVE BEEN WARNED.
*/
######## v.1.0.0
function top_login_box($email_login, $email_login_expires, $bbs_login, $bbs_public_key) {
global $webmail_action, $reactorcore, $webmail_login, $webmail_inbox, $webmail_logout, $userlang;

	if (email_login_confirm($email_login, $email_login_expires) && bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//both reactor user and webmail are logged in
		?>
<p align=right><b>[<a href="<?php echo $webmail_inbox;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $reactorcore; ?>/logout.php"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
	else if (email_login_confirm($email_login, $email_login_expires) && !(bbs_login_confirm($bbs_login, $bbs_public_key))) {
		//webmail is logged in, but reactor user isn't
		?>
<p align=right><b>[<a href="<?php echo $webmail_inbox;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $webmail_logout;?>"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
	else if (!(email_login_confirm($email_login, $email_login_expires)) && bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//reactor user is logged in but not webmail
		?>
<p align=right><b>[<a href="<?php echo $webmail_login;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $reactorcore; ?>/logout.php"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
	else {
		//no login whatsoever.  display login form for webmail.  CUSTOMIZE THIS
		?>
<!-- MAIL FORM -->
<FORM method="post" action="<?php echo $webmail_action;?>" enctype="application/x-www-form-urlencoded" name="login">
<SCRIPT type="text/javascript">
<!--//
function get_session () {
    today = new Date();
    document.login.sessionID.value = today.getTime();
}
//-->
</script>
<input type=hidden name=sessionID value="<?php echo time();?>">
<input type=hidden name=type value="phpreactor">
<INPUT TYPE=HIDDEN NAME=passed VALUE="login_parse"><INPUT TYPE="HIDDEN" NAME="first" VALUE="1">
<b>username:&nbsp;</b><INPUT type=text name=user_name size=8>&nbsp;&nbsp;
<b>hostname:&nbsp;</b><INPUT type=text name=hostname size=8 value="">&nbsp;&nbsp;
<b>password:&nbsp;</b><INPUT type=password name=password size=8 onChange="get_session();">&nbsp;
<INPUT class=b type=submit value="<?php echo picklang($userlang["check_email"]); ?>">&nbsp;</FORM>
<!-- END MAIL FORM -->
		<?php
	}
}

/*
* int_login_status() - this function is like the above top_login_status, except
* that it is designed for use with int.tpl.php - that is, ofr interior pages of
* the site.  if the user is logged in, it displays the appropriate links, if not
* nothing.
*/
######## v.1.0.0
function int_login_status($email_login, $email_login_expires, $bbs_login, $bbs_public_key) {
global $reactorcore, $webmail_login, $webmail_inbox, $webmail_logout, $userlang;
	if (email_login_confirm($email_login, $email_login_expires) && bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//both webmail and reactor user are logged in
		?>
<p align=right><b>[<a href="<?php echo $webmail_inbox;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $reactorcore; ?>/logout.php"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
	else if (email_login_confirm($email_login, $email_login_expires) && !(bbs_login_confirm($bbs_login, $bbs_public_key))) {
		//webmail logged in, reactor user isn't
		?>
<p align=right><b>[<a href="<?php echo $webmail_inbox;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $webmail_logout;?>"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
	else if (!(email_login_confirm($email_login, $email_login_expires)) && bbs_login_confirm($bbs_login, $bbs_public_key)) {
		//reactor user logged in, webmail isn't
		?>
<p align=right><b>[<a href="<?php echo $webmail_login;?>"><?php echo picklang($userlang["check_email"]); ?></a>]&nbsp;&nbsp;[<a href="<?php echo $reactorcore; ?>/logout.php"><?php echo picklang($userlang["logout"]); ?></a>]&nbsp;</b></p>		
		<?php
	}
}
?>
